UNIT QCard2;

(*                                                                      *)
(* AUTHOR: Michael G. Slack                    DATE WRITTEN: 2001/03/02 *)
(* ENVIRONMENT: Sibyl                                                   *)
(*                                                                      *)
(* Unit defines the interfaces and constants for working with the QCard2*)
(* DLL.  In addition, a simple 'card-deck' structure is defined and     *)
(* manipulated here.                                                    *)
(* QCard is based on a DLL release to the public created by Stephan     *)
(* Murphy with images from Daniel Di Bacco.                             *)
(*                                                                      *)
(* -------------------------------------------------------------------- *)
(*                                                                      *)
(* REVISED: yyyy/mm/dd -                                                *)
(*                                                                      *)

INTERFACE

 USES Graphics;

{$I g:\sibyl\progs\qcard2\QCard.inc} // contains the qcard constants and types

(************************************************************************)

 IMPORTS // from qcard2.dll

  FUNCTION  QC_GetCard(CardNum : LONGWORD) : TBITMAP; APIENTRY;
   'QCARD2' NAME 'QC_GETCARD';
      (* Function used to return a card image from the dll (1-52 or 1-104) *)

  FUNCTION  QC_GetCardBack(BackNum : LONGWORD) : TBITMAP; APIENTRY;
   'QCARD2' NAME 'QC_GETCARDBACK';
      (* Function used to return one of the dll's card back images *)

  FUNCTION  QC_GetCardSymbol(SymbolNum : LONGWORD) : TBITMAP; APIENTRY;
   'QCARD2' NAME 'QC_GETCARDSYMBOL';
      (* Function used to return one of the dll's card size symbol images *)

  FUNCTION  QC_GetJoker(JokerNum : LONGWORD) : TBITMAP; APIENTRY;
   'QCARD2' NAME 'QC_GETJOKER';
      (* Function used to return either the red or black joker *)

  FUNCTION  QC_GetCardColor(C_Num : LONGWORD) : INTEGER; APIENTRY;
   'QCARD2' NAME 'QC_GETCARDCOLOR';
      (* Function to return the color of the card in question *)

  FUNCTION  QC_GetCardSuit(C_Num : LONGWORD) : INTEGER; APIENTRY;
   'QCARD2' NAME 'QC_GETCARDSUIT';
      (* Function to return the suit of a card *)

  FUNCTION  QC_GetCardValue(C_Num : LONGWORD) : INTEGER; APIENTRY;
   'QCARD2' NAME 'QC_GETCARDVALUE';
      (* Function to get the value of a card (1-13, jack=11, queen=12, king=13) *)

  FUNCTION  QC_GetCV_Face10(C_Num : LONGWORD) : INTEGER; APIENTRY;
      (* Function to get the value of a card (1-10, jack,queen,king=10 also) *)
   'QCARD2' NAME 'QC_GETCV_FACE10';

  FUNCTION  QC_GetCardName(C_Num : LONGWORD) : STRING; APIENTRY;
   'QCARD2' NAME 'QC_GETCARDNAME';
      (* Function to return a string representing the card sent in *)

  PROCEDURE QC_ShuffleDeck(VAR Deck : Q_CARDDECK; NumDecks : INTEGER); APIENTRY;
   'QCARD2' NAME 'QC_SHUFFLEDECK';
      (* Procedure used to setup and shuffle a deck of cards *)

  FUNCTION  QC_GetNextCard(VAR Deck : Q_CARDDECK) : LONGWORD; APIENTRY;
   'QCARD2' NAME 'QC_GETNEXTCARD';
      (* Function used to get the next card (num) from the card deck *)
      (*  - returns 0 if no more cards are available                 *)

  FUNCTION  QC_GetNumCardsLeft(Deck : Q_CARDDECK) : INTEGER; APIENTRY;
   'QCARD2' NAME 'QC_GETNUMCARDSLEFT';
      (* Function returns the number of cards available in the card deck *)

 END; // end of imported functions

(************************************************************************)

IMPLEMENTATION

(************************************************************************)

INITIALIZATION (*of unit*)
 Randomize;
END. (*of unit*)
