UNIT QCardU;

(*                                                                      *)
(* AUTHOR: Michael G. Slack                    DATE WRITTEN: 2001/03/12 *)
(* ENVIRONMENT: Sibyl                                                   *)
(*                                                                      *)
(* Unit defines the 'meat' of the code to get and manipulate card       *)
(* images.  Included are routines to status cards and maintain card     *)
(* decks.                                                               *)
(* NOTE: Bitmap resources 1 - 499 are linked in by this unit.  These    *)
(* resources should not be used by the application.                     *)
(* QCard is based on a DLL release to the public created by Stephan     *)
(* Murphy with images from Daniel Di Bacco.                             *)
(*                                                                      *)
(* -------------------------------------------------------------------- *)
(*                                                                      *)
(* REVISED: 2003-01-17 - Rebuilt the card images in order to get them   *)
(*                       to display under SDD/Snap drivers.  Sibyl and  *)
(*                       24-bit BMPs have issues...                     *)
(*                                                                      *)

INTERFACE

 USES Graphics;

{$I g:\sibyl\progs\qcard2\QCard.inc} // contains the qcard constants and types

(************************************************************************)

 FUNCTION  QC_GetCard(CardNum : LONGWORD) : TBITMAP; APIENTRY;
     (* Function used to return a card image from the dll (1-52 or 1-104) *)

 FUNCTION  QC_GetCardBack(BackNum : LONGWORD) : TBITMAP; APIENTRY;
     (* Function used to return one of the dll's card back images *)

 FUNCTION  QC_GetCardSymbol(SymbolNum : LONGWORD) : TBITMAP; APIENTRY;
     (* Function used to return one of the dll's card size symbol images *)

 FUNCTION  QC_GetJoker(JokerNum : LONGWORD) : TBITMAP; APIENTRY;
     (* Function used to return either the red or black joker *)

 FUNCTION  QC_GetCardColor(C_Num : LONGWORD) : INTEGER; APIENTRY;
     (* Function to return the color of the card in question *)

 FUNCTION  QC_GetCardSuit(C_Num : LONGWORD) : INTEGER; APIENTRY;
     (* Function to return the suit of a card *)

 FUNCTION  QC_GetCardValue(C_Num : LONGWORD) : INTEGER; APIENTRY;
     (* Function to get the value of a card (1-13, jack=11, queen=12, king=13) *)

 FUNCTION  QC_GetCV_Face10(C_Num : LONGWORD) : INTEGER; APIENTRY;
     (* Function to get the value of a card (1-10, jack,queen,king=10 also) *)

 FUNCTION  QC_GetCardName(C_Num : LONGWORD) : STRING; APIENTRY;
     (* Function to return a string representing the card sent in *)

 PROCEDURE QC_ShuffleDeck(VAR Deck : Q_CARDDECK; NumDecks : INTEGER); APIENTRY;
     (* Procedure used to setup and shuffle a deck (or decks) of cards *)

 FUNCTION  QC_GetNextCard(VAR Deck : Q_CARDDECK) : LONGWORD; APIENTRY;
     (* Function used to get the next card (num) from the card deck *)
     (*  - returns 0 if no more cards are available                 *)

 FUNCTION  QC_GetNumCardsLeft(Deck : Q_CARDDECK) : INTEGER; APIENTRY;
     (* Function returns the number of cards available in the card deck *)

(************************************************************************)

IMPLEMENTATION

{$IFDEF OS2}
 {$R g:\sibyl\progs\qcard2\qcard.srf} // bitmap images
{$ELSE}
 {$R g:\sibyl\progs\qcard2\qcardw.srw}
{$ENDIF}

(************************************************************************)

 FUNCTION LoadBMResource(ResId : LONGWORD) : TBITMAP;
     (* function to load the bitmap resource with the given id *)

  BEGIN (*loadbmresource*)
   Result.Create;
   Result.LoadFromResourceId(ResId);
  END; (*loadbmresource*)

(************************************************************************)

 FUNCTION CheckMultiDeckValue(C_Num : LONGWORD) : LONGWORD;
     (* function to check if using multiple decks and back down to 1-52 *)
     (* - currently only support 3 decks.                               *)

  BEGIN (*checkmultideckvalue*)
   Result := C_Num;
   IF (C_Num > Max_CardNum) AND (C_Num <= Max_CardNum_2Deck)
    THEN Result := C_Num - Max_CardNum
   ELSE IF (C_Num > Max_CardNum_2Deck) AND (C_Num <= Max_CardNum_3Deck)
         THEN Result := C_Num - Max_CardNum_2Deck;
  END; (*checkmultideckvalue*)

(************************************************************************)

 FUNCTION QC_GetCard(CardNum : LONGWORD) : TBITMAP; APIENTRY;
     (* function to load a card out of the resources (1-52 [104/156]) *)

  BEGIN (*qc_getcard*)
   Result := NIL;
   IF (CardNum >= Min_CardNum) AND (CardNum <= Max_CardNum_3Deck)
    THEN BEGIN {load it from the resources then}
          Result := LoadBMResource(CheckMultiDeckValue(CardNum));
         END; {then}
  END; (*qc_getcard*)

(************************************************************************)

 FUNCTION QC_GetCardBack(BackNum : LONGWORD) : TBITMAP; APIENTRY;
     (* function to load a card back picture resource (200-205 currently) *)

  BEGIN (*qc_getcardback*)
   Result := NIL;
   IF (BackNum >= Min_CardBackNum) AND (BackNum <= Max_CardBackNum)
    THEN Result := LoadBMResource(BackNum);
  END; (*qc_getcardback*)

(************************************************************************)

 FUNCTION QC_GetCardSymbol(SymbolNum : LONGWORD) : TBITMAP; APIENTRY;
     (* function to load a symbol image to use with the card images (300-302) *)

  BEGIN (*qc_getcardsymbol*)
   Result := NIL;
   IF (SymbolNum >= Min_SymbolNum) AND (SymbolNum <= Max_SymbolNum)
    THEN Result := LoadBMResource(SymbolNum);
  END; (*qc_getcardsymbol*)

(************************************************************************)

 FUNCTION QC_GetJoker(JokerNum : LONGWORD) : TBITMAP; APIENTRY;
     (* function to get one of the joker images from the system (400,401) *)

  BEGIN (*qc_getjoker*)
   Result := NIL;
   IF (JokerNum = Red_Joker) OR (JokerNum = Black_Joker) // only two
    THEN Result := LoadBMResource(JokerNum);
  END; (*qc_getjoker*)

(************************************************************************)

 FUNCTION QC_GetCardColor(C_Num : LONGWORD) : INTEGER; APIENTRY;
     (* function to return red (0) or black (1) or -1 if invalid *)

  BEGIN (*qc_getcardcolor*)
   Result := q_Invalid_Return;

   C_Num := CheckMultiDeckValue(C_Num);

   IF ((C_Num >= 14) AND (C_Num <= 39)) OR (C_Num = 400)
    THEN Result := qc_Red_Card
   ELSE IF ((C_Num >= 1) AND (C_Num <= 13)) OR
           ((C_Num >= 40) AND (C_Num <= 52)) OR
           (C_Num = 401)
         THEN Result := qc_Black_Card;
  END; (*qc_getcardcolor*)

(************************************************************************)

 FUNCTION QC_GetCardSuit(C_Num : LONGWORD) : INTEGER; APIENTRY;
     (* function to return the suit of the card                  *)
     (* 0=clubs, 1=diamonds, 2=hearts, 3=spades, -1=invalid card *)

  BEGIN (*qc_getcardsuit*)
   Result := q_Invalid_Return;

   C_Num := CheckMultiDeckValue(C_Num);

   IF (C_Num >= 1) AND (C_Num <= 13)
    THEN Result := qs_Clubs
   ELSE IF (C_Num >= 14) AND (C_Num <= 26)
         THEN Result := qs_Diamonds
        ELSE IF (C_Num >= 27) AND (C_Num <= 39)
              THEN Result := qs_Hearts
             ELSE IF (C_Num >= 40) AND (C_Num <= 52)
                   THEN Result := qs_Spades;
  END; (*qc_getcardsuit*)

(************************************************************************)

 FUNCTION QC_GetCardValue(C_Num : LONGWORD) : INTEGER; APIENTRY;
     (* function to return the card value (1 - 13 or -1 if invalid) *)

  BEGIN (*qc_getcardvalue*)
   Result := q_Invalid_Return;

   C_Num := CheckMultiDeckValue(C_Num);

   IF (C_Num >= 40) AND (C_Num <= 52)
    THEN C_Num := C_Num - 39
   ELSE IF (C_Num >= 27) AND (C_Num <= 39)
         THEN C_Num := C_Num - 26
        ELSE IF (C_Num >= 14) AND (C_Num <= 26)
              THEN C_Num := C_Num - 13;

   IF (C_Num >= 1) AND (C_Num <= 13) THEN Result := C_Num;
  END; (*qc_getcardvalue*)

(************************************************************************)

 FUNCTION QC_GetCV_Face10(C_Num : LONGWORD) : INTEGER; APIENTRY;
     (* function to return the card value (1 - 10 or -1 if invalid) *)

  BEGIN (*qc_getcv_face10*)
   Result := QC_GetCardValue(C_Num);
   IF Result > 10 THEN Result := 10;
  END; (*qc_getcv_face10*)

(************************************************************************)

 FUNCTION QC_GetCardName(C_Num : LONGWORD) : STRING; APIENTRY;
     (* function to return a string containing the name of the card *)

    VAR I_Suit, I_Value : INTEGER;

  BEGIN (*qc_getcardname*)
   Result := 'Invalid Card';

   IF (C_Num >= Min_CardBackNum) AND (C_Num <= Max_CardBackNum)
    THEN Result := 'Card Back'
   ELSE IF (C_Num >= Min_SymbolNum) AND (C_Num <= Max_SymbolNum)
         THEN Result := 'Symbol Card'
        ELSE IF (C_Num = Red_Joker) OR (C_Num = Black_Joker)
              THEN Result := 'Joker'
             ELSE BEGIN {get suit and value and build name}
                   I_Suit := QC_GetCardSuit(C_Num);
                   I_Value := QC_GetCardValue(C_Num);
                   IF (I_Suit <> q_Invalid_Return) AND
                      (I_Value <> q_Invalid_Return)
                    THEN BEGIN {do it}
                          CASE I_Value OF // get value of
                           qv_Ace   : Result := 'Ace of ';
                           qv_Two   : Result := 'Two of ';
                           qv_Three : Result := 'Three of ';
                           qv_Four  : Result := 'Four of ';
                           qv_Five  : Result := 'Five of ';
                           qv_Six   : Result := 'Six of ';
                           qv_Seven : Result := 'Seven of ';
                           qv_Eight : Result := 'Eight of ';
                           qv_Nine  : Result := 'Nine of ';
                           qv_Ten   : Result := 'Ten of ';
                           qv_Jack  : Result := 'Jack of ';
                           qv_Queen : Result := 'Queen of ';
                           qv_King  : Result := 'King of ';
                          END; {case}
                          CASE I_Suit OF // get kind now
                           qs_Clubs    : Result := Result + 'Clubs';
                           qs_Diamonds : Result := Result + 'Diamonds';
                           qs_Hearts   : Result := Result + 'Hearts';
                           qs_Spades   : Result := Result + 'Spades';
                          END; {case}
                         END; {then}
                  END; {else}
  END; (*qc_getcardname*)

(************************************************************************)

 PROCEDURE QC_ShuffleDeck(VAR Deck : Q_CARDDECK; NumDecks : INTEGER); APIENTRY;
     (* procedure use to initialize and shuffle a deck (or decks) of cards *)

    VAR I, J, K : INTEGER;
        TT      : LONGWORD;

  BEGIN (*qc_shuffledeck*)
   // initialize
   Deck.NextCard := Min_CardNum;
   CASE NumDecks OF
    qcn_2Deck : Deck.LastCard := Max_CardNum_2Deck;
    qcn_3Deck : Deck.LastCard := Max_CardNum_3Deck;
    ELSE Deck.LastCard := Max_CardNum; // default to one deck
   END; {case}
   FOR I := Deck.NextCard TO Deck.LastCard DO
    Deck.Cards[I] := I;
   // now shuffle
   FOR I := 1 TO 10 DO
    FOR J := 1 TO Deck.LastCard DO
     BEGIN {do it}
      K := Random(Deck.LastCard) + 1;
      // swap cards between j and k
      TT := Deck.Cards[J];
      Deck.Cards[J] := Deck.Cards[K];
      Deck.Cards[K] := TT;
     END; {j for}
  END; (*qc_shuffledeck*)

(************************************************************************)

 FUNCTION QC_GetNextCard(VAR Deck : Q_CARDDECK) : LONGWORD; APIENTRY;
     (* function to return the value of the next card from the deck *)

  BEGIN (*qc_getnextcard*)
   IF Deck.NextCard > Deck.LastCard
    THEN Result := q_NoMoreCardsInDeck
   ELSE BEGIN {return next card}
         Result := Deck.Cards[Deck.NextCard];
         Deck.NextCard := Deck.NextCard + 1;
        END; {else}
  END; (*qc_getnextcard*)

(************************************************************************)

 FUNCTION QC_GetNumCardsLeft(Deck : Q_CARDDECK) : INTEGER; APIENTRY;
     (* function to get the number of cards left in the deck(s) *)

  BEGIN (*qc_getnumcardsleft*)
   Result := Deck.LastCard - Deck.NextCard + 1;
  END; (*qc_getnumcardsleft*)

(************************************************************************)

INITIALIZATION (*of unit*)
 Randomize;
END. (*of unit*)
